#ifndef DIFMAP_H
#define DIFMAP_H

#include <t3d/t3d.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Nb of classes to generate the probas associated to a difficulty */
/*  For the moment fixed to 3: Flat/Rough/Obstacle terrain */
#define DIFMAP_NB_CLASSES 3

typedef struct DIFMAP_CELL {
  
  int     nbFused;
  short   firstDate;   /* First update (fusion) date for this cell */
  short   lastDate;    /* Last update (fusion) date */

  double  difficulty;
  double  sigma;       /* standard deviation associated to the difficulty */

  double  probas[DIFMAP_NB_CLASSES];

  double confidence;   /* confidence on difficulty found (depends on the confidence on the predicted configurations */

} DIFMAP_CELL;


/*--- Type of Fusion to be used for the difficulty array  ---*/
typedef enum DIFMAP_FUSION_METHOD {
  FUSION_WORST_CASE,   /* Keep the worst difficulty */
  FUSION_AVERAGE       /* Make an average of the difficulties */
} DIFMAP_FUSION_METHOD;



/******************************************************************************/
/******************************************************************************/
/*** Difficulty map ******/

typedef struct DIF_MAP {
  int		nbLines;
  int		nbCols;		/* Nb of lines and columns of the map */

  int           nbBorderLines;  /* Nb of lines on the border where difficulty cannot be computed (corresponds to half size of the robot) */
  int           nbBorderCols;   /* Id. with nb of cols */

  float		xOrigin;
  float		yOrigin;	/* Coordinates of the topleft corner of the
				   (0,0) pixel in the global frame */

  T3D*          mainToOrigin;   /* Position of the (0,0) point of the map wrt the global frame */

  float		xScale;		/* Size of a cell in meters */
  float         yScale;          /* -> should have xScale = yScale */

  int           deltaLines;     /* Discretization rate wrt dem map */
  int           deltaCols;

  DIFMAP_FUSION_METHOD  fusionMethod;
  int           nbFusions;     /* Number of fusions already made in this difmap */
  int           lastDate;      /* Date of the last fusion made */

  /* double*       difficulty; */
  DIFMAP_CELL*  cells;  /* cell tab (set as a C array, a ima1 or DEM's cells tab  */

} DIF_MAP;


/* macro to get a particular difmap cell, at line i and col j */
#define difmap_one_cell(a,i,j) (((a)->cells) + ((i) * ((a)->nbCols)) + (j))

#ifdef __cplusplus
}
#endif

#endif /* DIFMAP_H */
